//
//  ViewController.swift
//  OCRtest
//
//  Created by Stinkie on 05.01.17.
//  Copyright © 2017 Stinkie. All rights reserved.
//

import UIKit
import Foundation

class ViewController: UIViewController, UITextViewDelegate, UINavigationControllerDelegate {

    @IBOutlet weak var textView: UITextView!
    @IBOutlet weak var findTextField: UITextField!
    @IBOutlet weak var replaceTextField: UITextField!
    @IBOutlet weak var topMarginConstraint: NSLayoutConstraint!

    var originalTopMargin:CGFloat!
    var activityIndicator:UIActivityIndicatorView!

    override func viewDidLoad() {
        super.viewDidLoad()
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        // Remember the original value of the constrain "Top Margin Constraint" in the view
        originalTopMargin = topMarginConstraint.constant
    }
    
    func scaleImage(image: UIImage, maxDimension: CGFloat) -> UIImage {
        var scaledSize = CGSize(width: maxDimension, height: maxDimension)
        var scaleFactor: CGFloat
        
        if (image.size.width > image.size.height) {
            scaleFactor = image.size.height / image.size.width
            scaledSize.width = maxDimension
            scaledSize.height = scaledSize.width * scaleFactor
        } else {
            scaleFactor = image.size.width / image.size.height
            scaledSize.height = maxDimension
            scaledSize.width = scaledSize.height * scaleFactor
        }
        
        UIGraphicsBeginImageContext(scaledSize)
        image.draw(in: CGRect(0, 0, scaledSize.width, scaledSize.height))
        let scaledImage = UIGraphicsGetImageFromCurrentImageContext()
        UIGraphicsEndImageContext()
        
        return scaledImage!
        
    }
    
    func performImageRecognition(image: UIImage) {
        // 1.
        // Initialize tesseract to a contain a new G8Tesseract object
        let tesseract = G8Tesseract()
        // 2.
        // Tesseract will search for the .traineddata files of the languages you specify in this parameter; specifying eng and fra will search for “eng.traineddata” and “fra.traineddata”containing the data to detect English and French text respectively. The French trained data has been included in this project since the sample poem you’ll be using for this tutorial contains a bit of French (Très romantique!). The poem’s French accented characters aren’t in the English character set, so you need to link to French .traineddata in order for those accents to appear; it’s also good to include the French data since there’s a component of .traineddata which takes language vocabulary into account
        tesseract.language = "eng+fra+deu"
        // 3.
        // You can specify three different OCR engine modes: .TesseractOnly, which is the fastest, but least accurate method; .CubeOnly, which is slower but more accurate since it employs more artificial intelligence; and .TesseractCubeCombined, which runs both .TesseractOnly and .CubeOnly to produce the most accurate results — but as a result is the slowest mode of the three
        tesseract.engineMode = .tesseractCubeCombined
        // 4.
        // Tesseract assumes by default that it’s processing a uniform block of text, but your sample image has multiple paragraphs. Tesseract’s pageSegmentationMode lets the Tesseract engine know how the text is divided, so in this case, set pageSegmentationMode to .Auto to allow for fully automatic page segmentation and thus the ability to recognize paragraph breaks
        tesseract.pageSegmentationMode = .auto
        // 5.
        // Here you set maximumRecognitionTime to limit the amount of time your Tesseract engine devotes to image recognition. However, only the Tesseract engine is limited by this setting; if you’re using the .CubeOnly or .TesseractCubeCombined engine mode, the Cube engine will continue processing even once your Tesseract engine has hit its maximumRecognitionTime
        tesseract.maximumRecognitionTime = 60.0
        // 6.
        // You’ll get the best results of Tesseract when the text contrasts highly with the background. Tesseract has a built in filter, g8_blackAndWhite(), that desaturates the image, increases the contrast, and reduces the exposure. Here, you’re assigning the filtered image to the image property of your Tesseract object, before kicking off the Tesseract image recognition process
        tesseract.image = image.g8_blackAndWhite()
        // 7.
        // Note that the image recognition is synchronous so at this point, the text is available. You then put the recognized text into your textView and make the view editable so your user can edit it as she likes
        textView.text = tesseract.recognizedText
        textView.isEditable = true
        // 8.
        // Finally, remove the activity indicator to signal that the OCR is complete and to let the user edit their poem
        removeActivityIndicator()
        
    }
    
    @IBAction func takePhoto(_ sender: AnyObject) {
        // 1. 
        // If you’re currently editing either the text view or a text field, close the keyboard and move the view back to its original position
        view.endEditing(true)
        moveViewDown()
        
        // 2. 
        // Create a UIAlertController with the action sheet style to present a set of capture options to the user
        let imagePickerActionSheet = UIAlertController(title: "Foto aufnehmen/hochladen", message: nil, preferredStyle: .actionSheet)
        
        
        // 3. 
        // If the device has a camera, add the Take Photo button to imagePickerActionSheet. Selecting this button creates and presents an instance of UIImagePickerController with sourceType .Camera.
        if UIImagePickerController.isSourceTypeAvailable(.camera) {
            let cameraButton = UIAlertAction(title: "Foto aufnehmen",
                                             style: .default) {
                (alert) -> Void in
                let imagePicker = UIImagePickerController()
                imagePicker.delegate = self
                imagePicker.sourceType = .camera
                self.present(imagePicker, animated: true, completion: nil)
            }
            imagePickerActionSheet.addAction(cameraButton)
        }
        
        // 4.
        // Add a Choose Existing button to imagePickerActionSheet. Selecting this button creates and presents an instance of UIImagePickerController with sourceType .PhotoLibrary
        let libraryButton = UIAlertAction(title: "Foto auswählen",
                                          style: .default) {
                (alert) -> Void in
                let imagePicker = UIImagePickerController()
                imagePicker.delegate = self
                imagePicker.sourceType = .photoLibrary
                self.present(imagePicker, animated: true, completion: nil)
        }
        imagePickerActionSheet.addAction(libraryButton)
        
        // 5.
        // Add a Cancel button to imagePickerActionSheet. Selecting this button cancels your UIImagePickerController, even though you don’t specify an action beyond setting the style as .Cancel
        let cancelButton = UIAlertAction(title: "Abbrechen", style: .cancel) {
            (alert) -> Void in
        }
        imagePickerActionSheet.addAction(cancelButton)
        
        // 6.
        // Finally, present your instance of UIAlertController
        present(imagePickerActionSheet, animated: true, completion: nil)
        
        print("Ende")
        
    }
    
    @IBAction func swapText(_ sender: AnyObject) {
        // 1. Only execute swap if textView, findText and replaceText not empty
        if textView.text.isEmpty {
            return
        }
        
        let findText = findTextField.text
        let replaceText = replaceTextField.text
        if !(findText?.isEmpty)! && !(replaceText?.isEmpty)! {
                // 2.
                // Replace all occurences of the string typed into findTextField in the textView with String in replaceTextField
                textView.text = textView.text.replacingOccurrences(of: findText!, with: replaceText!)
                // 3.
                // Clear fields after replace
                findTextField = nil
                replaceTextField = nil
                // 4.
                // Finally, resign the keyboard and move the view back in the current position
                view.endEditing(true)
                moveViewDown()
        }
    }
    
    @IBAction func sharePoem(_ sender: AnyObject) {
        // 1.
        // if textview is empty don't share anything
        if textView.text.isEmpty {
            return
        }
        
        // 2.
        // create a new instance of UIActivityControlle, put the text from the text view inside an array and pass it in as activity item to be shared
        let activityViewController = UIActivityViewController(activityItems: [textView.text], applicationActivities: nil)
        // 3.
        // UIActivityViewController has a long list of build-in activity types. ou can exclude UIActivityTypeAssignToContact, UIActivityTypeSaveToCameraRoll, UIActivityTypeAddToReadingList, UIActivityTypePostToFlickr, and UIActivityTypePostToVimeo since they don’t make much sense in this context
        let excludeActivities = [
          UIActivityType.assignToContact,
          UIActivityType.saveToCameraRoll,
          UIActivityType.addToReadingList,
          UIActivityType.postToFlickr,
          UIActivityType.postToVimeo]
        
        activityViewController.excludedActivityTypes = excludeActivities
        // 4.
        // Finally present your UIActivityViewController
        
        present(activityViewController, animated: true, completion: nil)
        
    }

    // Activity Indicator methods
    
    func addActivityIndicator() {
        activityIndicator = UIActivityIndicatorView(frame: view.bounds)
        activityIndicator.activityIndicatorViewStyle = .whiteLarge
        activityIndicator.backgroundColor = UIColor(white: 0, alpha: 0.25)
        activityIndicator.startAnimating()
        view.addSubview(activityIndicator)
    }
    
    func removeActivityIndicator() {
        activityIndicator.removeFromSuperview()
        activityIndicator = nil
    }
    
    
    // The remaining methods handle the keyboard resignation/
    // move the view so that the first responders aren't hidden
    
    func moveViewUp() {
        if topMarginConstraint.constant != originalTopMargin {
            return
        }
        
        topMarginConstraint.constant -= 135
        UIView.animate(withDuration: 0.3, animations: { () -> Void in
            self.view.layoutIfNeeded()
        })
    }
    
    func moveViewDown() {
        if topMarginConstraint.constant == originalTopMargin {
            return
        }
        
        topMarginConstraint.constant = originalTopMargin
        UIView.animate(withDuration: 0.3, animations: { () -> Void in
            self.view.layoutIfNeeded()
        })
        
    }
    
    @IBAction func backgroundTapped(_ sender: AnyObject) {
        view.endEditing(true)
        moveViewDown()
    }
}

extension ViewController: UITextFieldDelegate {
    func textFieldDidBeginEditing(_ textField: UITextField) {
        moveViewUp()
    }
    
    @IBAction private func textFieldEndEditing(_ sender: AnyObject) {
        view.endEditing(true)
        moveViewDown()
    }
    
    private func textViewDidBeginEditing(_ textView: UITextView) {
        moveViewDown()
    }
    
}

extension ViewController: UIImagePickerControllerDelegate {
    func imagePickerController(_ picker: UIImagePickerController, didFinishPickingMediaWithInfo info: [String : Any]) {
        let selectedPhoto = info[UIImagePickerControllerOriginalImage] as! UIImage
        let scaledImage = scaleImage(image: selectedPhoto, maxDimension: 640)
        
        addActivityIndicator()
        
        dismiss(animated: true, completion: {
            self.performImageRecognition(image: scaledImage)
        })
    }
}

extension CGRect{
    init(_ x:CGFloat,_ y:CGFloat,_ width:CGFloat,_ height:CGFloat) {
        self.init(x:x,y:y,width:width,height:height)
    }
    
}
